/*
 * Copyright 2009 Mozilla Corporation.
 *
 * This is part of HarfBuzz, an OpenType Layout engine library.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Created by Jonathan Kew on 18/06/2009.
 */

#ifndef HB_SCRIPT_CODE_MAP_H
#define HB_SCRIPT_CODE_MAP_H

/* map script codes returned by hb_script_run_next to OpenType shapers and script tags */

#include "hb-common.h"
#include "hb-shaper-private.h"
#include "hb-unicode-properties.h"

struct _hb_script_code_mapping_t {
  hb_script_shaper_func_t shaping_function;
  void * shaping_data;
  hb_tag_t script_tag;
};
typedef struct _hb_script_code_mapping_t hb_script_code_mapping_t;

HB_BEGIN_DECLS

/*
 * Map a hb_script_code_t value returned by the script itemizer
 * to a script mapping record.
 */
const hb_script_code_mapping_t *
hb_get_script_code_mapping(hb_script_code_t script_code);

/*
 * Some scripts have both an old "legacy" tag and a new one.
 * We first map to the new tag, but if that fails, this function will
 * return the old tag for us to try (or HB_SCRIPT_TAG_DEFAULT if there
 * is no old script tag).
 */
hb_tag_t
hb_get_old_script_tag(hb_tag_t new_tag);

HB_END_DECLS

#endif
