/*
 * Copyright 2009 Mozilla Corporation.
 *
 * This is part of HarfBuzz, an OpenType Layout engine library.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Created by Jonathan Kew on 23/06/2009.
 */

#include "hb-unicode-properties.h"
#include "hb-private.h"

#ifdef HB_USE_ICU_PROPERTIES

#include "uscript.h"

/* This is implemented as a macro in the header
hb_bidi_class_t
hb_get_bidi_class(hb_codepoint_t ch)
{
  return (hb_bidi_class_t)u_charDirection(ch);
}
*/

hb_arabic_join_type_t
hb_get_arabic_join_type(hb_codepoint_t ch)
{
  hb_arabic_join_type_t t = (hb_arabic_join_type_t)u_getIntPropertyValue(ch, UCHAR_JOINING_TYPE);
  /* our shaper requires a distinction for Syriac ALAPH and DALATH/RISH */
  if (HB_UNLIKELY(t == HB_JOIN_RGHT)) {
    if (HB_UNLIKELY(ch == 0x0710)) {
      return HB_JOIN_ALPH;
    }
    if (HB_UNLIKELY(ch == 0x0715 || ch == 0x0716 || ch == 0x072A || ch == 0x072F)) {
      return HB_JOIN_DLRS;
    }
  }
  return t;
}

hb_script_code_t
hb_get_script_code(hb_codepoint_t ch)
{
  UScriptCode sc = (UScriptCode)u_getIntPropertyValue(ch, UCHAR_SCRIPT);
  if (HB_UNLIKELY(sc == USCRIPT_INVALID_CODE)) {
    return HB_SCRIPT_UNKNOWN;
  }
  return (hb_script_code_t)sc; /* NB: ICU and HB enumerations correspond code-for-code! */
}

#else

#include "hb-unicode-property-data.c"

#define GET_JOIN_TYPE(v)    ((v).type_class >> 5)
#define GET_BIDI_CLASS(v)   ((v).type_class & 0x1f)
#define GET_SCRIPT_CODE(v)  ((v).script)

hb_arabic_join_type_t
hb_get_arabic_join_type(hb_codepoint_t ch)
{
  if (HB_LIKELY(ch < 0x10000)) {
    return GET_JOIN_TYPE(values[pages[0][ch >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
  }
  if (HB_UNLIKELY(ch >= 0x110000)) {
    return HB_JOIN_NONE;
  }
  return GET_JOIN_TYPE(values[pages[planes[(ch >> 16) - 1]][(ch & 0xffff) >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
}

hb_bidi_class_t
hb_get_bidi_class(hb_codepoint_t ch)
{
  if (HB_LIKELY(ch < 0x10000)) {
    return GET_BIDI_CLASS(values[pages[0][ch >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
  }
  if (HB_UNLIKELY(ch >= 0x110000)) {
    return HB_BIDI_L;
  }
  return GET_BIDI_CLASS(values[pages[planes[(ch >> 16) - 1]][(ch & 0xffff) >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
}

hb_script_code_t
hb_get_script_code(hb_codepoint_t ch)
{
  if (HB_LIKELY(ch < 0x10000)) {
    return GET_SCRIPT_CODE(values[pages[0][ch >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
  }
  if (HB_UNLIKELY(ch >= 0x110000)) {
    return HB_SCRIPT_UNKNOWN;
  }
  return GET_SCRIPT_CODE(values[pages[planes[(ch >> 16) - 1]][(ch & 0xffff) >> CHAR_BITS]][ch & ((1 << CHAR_BITS) - 1)]);
}

#endif
