/*
 * Copyright 2009 Mozilla Corporation.
 *
 * This is part of HarfBuzz, an OpenType Layout engine library.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Created by Jonathan Kew on 23/06/2009.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Script code assignments below are copied directly from ICU,
 * to allow either HB_SCRIPT_... or USCRIPT_... codes to index the
 * the same array in hb-script-code-map.c.
 *
 * The original ICU license of the code follows:
 *
 * ICU License - ICU 1.8.1 and later
 *
 * COPYRIGHT AND PERMISSION NOTICE
 * 
 * Copyright (c) 1995-2009 International Business Machines Corporation and
 * others
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, provided that the above copyright notice(s) and this
 * permission notice appear in all copies of the Software and that both the
 * above copyright notice(s) and this permission notice appear in supporting
 * documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE
 * BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of a copyright holder shall
 * not be used in advertising or otherwise to promote the sale, use or other
 * dealings in this Software without prior written authorization of the
 * copyright holder.
 *
 * All trademarks and registered trademarks mentioned herein are the property
 * of their respective owners. 
 */

#ifndef HB_UNICODE_PROPERTIES_H
#define HB_UNICODE_PROPERTIES_H

/*
 * Unicode character properties needed by HarfBuzz; can be built to call ICU
 * or to use our own compiled-in data.
 */

/*
 * Define to get character properties from ICU instead of our built-in data;
 * recommended if the client app is linking with ICU anyway for other reasons.
 */
#undef HB_USE_ICU_PROPERTIES

#include "hb-common.h"

/* Note that values from NONE to TRAN match UJoiningType values from ICU */
enum _hb_arabic_join_type_t {
  HB_JOIN_NONE = 0,
  HB_JOIN_CAUS = 1,
  HB_JOIN_DUAL = 2,
  HB_JOIN_LEFT = 3,
  HB_JOIN_RGHT = 4,
  HB_JOIN_TRAN = 5,
  HB_JOIN_ALPH = 6,
  HB_JOIN_DLRS = 7,
  HB_NUM_JOIN_TYPES = 8
};
typedef enum _hb_arabic_join_type_t hb_arabic_join_type_t;

/* Note that these class values match UCharDirection constants from ICU */
enum _hb_bidi_class_t {
  HB_BIDI_L = 0,
  HB_BIDI_R = 1,
  HB_BIDI_EN = 2,
  HB_BIDI_ES = 3,
  HB_BIDI_ET = 4,
  HB_BIDI_AN = 5,
  HB_BIDI_CS = 6,
  HB_BIDI_B = 7,
  HB_BIDI_S = 8,
  HB_BIDI_WS = 9,
  HB_BIDI_ON = 10,
  HB_BIDI_LRE = 11,
  HB_BIDI_LRO = 12,
  HB_BIDI_AL = 13,
  HB_BIDI_RLE = 14,
  HB_BIDI_RLO = 15,
  HB_BIDI_PDF = 16,
  HB_BIDI_NSM = 17,
  HB_BIDI_BN = 18,
  HB_NUM_BIDI_CLASSES = 19
};
typedef enum _hb_bidi_class_t hb_bidi_class_t;

/* These values match assignments in ICU's uscript.h */
enum _hb_script_code_t {
  HB_SCRIPT_INVALID_CODE = -1,
  HB_SCRIPT_COMMON       =  0 , /* Zyyy */
  HB_SCRIPT_INHERITED    =  1,  /* Qaai */
  HB_SCRIPT_ARABIC       =  2,  /* Arab */
  HB_SCRIPT_ARMENIAN     =  3,  /* Armn */
  HB_SCRIPT_BENGALI      =  4,  /* Beng */
  HB_SCRIPT_BOPOMOFO     =  5,  /* Bopo */
  HB_SCRIPT_CHEROKEE     =  6,  /* Cher */
  HB_SCRIPT_COPTIC       =  7,  /* Copt */
  HB_SCRIPT_CYRILLIC     =  8,  /* Cyrl */
  HB_SCRIPT_DESERET      =  9,  /* Dsrt */
  HB_SCRIPT_DEVANAGARI   = 10,  /* Deva */
  HB_SCRIPT_ETHIOPIC     = 11,  /* Ethi */
  HB_SCRIPT_GEORGIAN     = 12,  /* Geor */
  HB_SCRIPT_GOTHIC       = 13,  /* Goth */
  HB_SCRIPT_GREEK        = 14,  /* Grek */
  HB_SCRIPT_GUJARATI     = 15,  /* Gujr */
  HB_SCRIPT_GURMUKHI     = 16,  /* Guru */
  HB_SCRIPT_HAN          = 17,  /* Hani */
  HB_SCRIPT_HANGUL       = 18,  /* Hang */
  HB_SCRIPT_HEBREW       = 19,  /* Hebr */
  HB_SCRIPT_HIRAGANA     = 20,  /* Hira */
  HB_SCRIPT_KANNADA      = 21,  /* Knda */
  HB_SCRIPT_KATAKANA     = 22,  /* Kana */
  HB_SCRIPT_KHMER        = 23,  /* Khmr */
  HB_SCRIPT_LAO          = 24,  /* Laoo */
  HB_SCRIPT_LATIN        = 25,  /* Latn */
  HB_SCRIPT_MALAYALAM    = 26,  /* Mlym */
  HB_SCRIPT_MONGOLIAN    = 27,  /* Mong */
  HB_SCRIPT_MYANMAR      = 28,  /* Mymr */
  HB_SCRIPT_OGHAM        = 29,  /* Ogam */
  HB_SCRIPT_OLD_ITALIC   = 30,  /* Ital */
  HB_SCRIPT_ORIYA        = 31,  /* Orya */
  HB_SCRIPT_RUNIC        = 32,  /* Runr */
  HB_SCRIPT_SINHALA      = 33,  /* Sinh */
  HB_SCRIPT_SYRIAC       = 34,  /* Syrc */
  HB_SCRIPT_TAMIL        = 35,  /* Taml */
  HB_SCRIPT_TELUGU       = 36,  /* Telu */
  HB_SCRIPT_THAANA       = 37,  /* Thaa */
  HB_SCRIPT_THAI         = 38,  /* Thai */
  HB_SCRIPT_TIBETAN      = 39,  /* Tibt */
  /** Canadian_Aboriginal script. @stable ICU 2.6 */
  HB_SCRIPT_CANADIAN_ABORIGINAL = 40,  /* Cans */
  /** Canadian_Aboriginal script (alias). @stable ICU 2.2 */
  HB_SCRIPT_UCAS         = HB_SCRIPT_CANADIAN_ABORIGINAL,
  HB_SCRIPT_YI           = 41,  /* Yiii */
  HB_SCRIPT_TAGALOG      = 42,  /* Tglg */
  HB_SCRIPT_HANUNOO      = 43,  /* Hano */
  HB_SCRIPT_BUHID        = 44,  /* Buhd */
  HB_SCRIPT_TAGBANWA     = 45,  /* Tagb */
  
  /* New scripts in Unicode 4 @stable ICU 2.6 */
  HB_SCRIPT_BRAILLE      = 46,  /* Brai */
  HB_SCRIPT_CYPRIOT      = 47,  /* Cprt */
  HB_SCRIPT_LIMBU        = 48,  /* Limb */
  HB_SCRIPT_LINEAR_B     = 49,  /* Linb */
  HB_SCRIPT_OSMANYA      = 50,  /* Osma */
  HB_SCRIPT_SHAVIAN      = 51,  /* Shaw */
  HB_SCRIPT_TAI_LE       = 52,  /* Tale */
  HB_SCRIPT_UGARITIC     = 53,  /* Ugar */
  
  /** New script code in Unicode 4.0.1 @stable ICU 3.0 */
  HB_SCRIPT_KATAKANA_OR_HIRAGANA = 54,/*Hrkt */
  
  /* New scripts in Unicode 4.1 @stable ICU 3.4 */
  HB_SCRIPT_BUGINESE      = 55, /* Bugi */
  HB_SCRIPT_GLAGOLITIC    = 56, /* Glag */
  HB_SCRIPT_KHAROSHTHI    = 57, /* Khar */
  HB_SCRIPT_SYLOTI_NAGRI  = 58, /* Sylo */
  HB_SCRIPT_NEW_TAI_LUE   = 59, /* Talu */
  HB_SCRIPT_TIFINAGH      = 60, /* Tfng */
  HB_SCRIPT_OLD_PERSIAN   = 61, /* Xpeo */
  
  /* New script codes from ISO 15924 @stable ICU 3.6 */
  HB_SCRIPT_BALINESE                      = 62, /* Bali */
  HB_SCRIPT_BATAK                         = 63, /* Batk */
  HB_SCRIPT_BLISSYMBOLS                   = 64, /* Blis */
  HB_SCRIPT_BRAHMI                        = 65, /* Brah */
  HB_SCRIPT_CHAM                          = 66, /* Cham */
  HB_SCRIPT_CIRTH                         = 67, /* Cirt */
  HB_SCRIPT_OLD_CHURCH_SLAVONIC_CYRILLIC  = 68, /* Cyrs */
  HB_SCRIPT_DEMOTIC_EGYPTIAN              = 69, /* Egyd */
  HB_SCRIPT_HIERATIC_EGYPTIAN             = 70, /* Egyh */
  HB_SCRIPT_EGYPTIAN_HIEROGLYPHS          = 71, /* Egyp */
  HB_SCRIPT_KHUTSURI                      = 72, /* Geok */
  HB_SCRIPT_SIMPLIFIED_HAN                = 73, /* Hans */
  HB_SCRIPT_TRADITIONAL_HAN               = 74, /* Hant */
  HB_SCRIPT_PAHAWH_HMONG                  = 75, /* Hmng */
  HB_SCRIPT_OLD_HUNGARIAN                 = 76, /* Hung */
  HB_SCRIPT_HARAPPAN_INDUS                = 77, /* Inds */
  HB_SCRIPT_JAVANESE                      = 78, /* Java */
  HB_SCRIPT_KAYAH_LI                      = 79, /* Kali */
  HB_SCRIPT_LATIN_FRAKTUR                 = 80, /* Latf */
  HB_SCRIPT_LATIN_GAELIC                  = 81, /* Latg */
  HB_SCRIPT_LEPCHA                        = 82, /* Lepc */
  HB_SCRIPT_LINEAR_A                      = 83, /* Lina */
  HB_SCRIPT_MANDAEAN                      = 84, /* Mand */
  HB_SCRIPT_MAYAN_HIEROGLYPHS             = 85, /* Maya */
  HB_SCRIPT_MEROITIC                      = 86, /* Mero */
  HB_SCRIPT_NKO                           = 87, /* Nkoo */
  HB_SCRIPT_ORKHON                        = 88, /* Orkh */
  HB_SCRIPT_OLD_PERMIC                    = 89, /* Perm */
  HB_SCRIPT_PHAGS_PA                      = 90, /* Phag */
  HB_SCRIPT_PHOENICIAN                    = 91, /* Phnx */
  HB_SCRIPT_PHONETIC_POLLARD              = 92, /* Plrd */
  HB_SCRIPT_RONGORONGO                    = 93, /* Roro */
  HB_SCRIPT_SARATI                        = 94, /* Sara */
  HB_SCRIPT_ESTRANGELO_SYRIAC             = 95, /* Syre */
  HB_SCRIPT_WESTERN_SYRIAC                = 96, /* Syrj */
  HB_SCRIPT_EASTERN_SYRIAC                = 97, /* Syrn */
  HB_SCRIPT_TENGWAR                       = 98, /* Teng */
  HB_SCRIPT_VAI                           = 99, /* Vaii */
  HB_SCRIPT_VISIBLE_SPEECH                = 100, /* Visp */
  HB_SCRIPT_CUNEIFORM                     = 101,/* Xsux */
  HB_SCRIPT_UNWRITTEN_LANGUAGES           = 102,/* Zxxx */
  HB_SCRIPT_UNKNOWN                       = 103,/* Zzzz */ /* Unknown="Code for uncoded script", for unassigned code points */
  
  /* New script codes from ISO 15924 @stable ICU 3.8 */
  HB_SCRIPT_CARIAN                        = 104,/* Cari */
  HB_SCRIPT_JAPANESE                      = 105,/* Jpan */
  HB_SCRIPT_LANNA                         = 106,/* Lana */
  HB_SCRIPT_LYCIAN                        = 107,/* Lyci */
  HB_SCRIPT_LYDIAN                        = 108,/* Lydi */
  HB_SCRIPT_OL_CHIKI                      = 109,/* Olck */
  HB_SCRIPT_REJANG                        = 110,/* Rjng */
  HB_SCRIPT_SAURASHTRA                    = 111,/* Saur */
  HB_SCRIPT_SIGN_WRITING                  = 112,/* Sgnw */
  HB_SCRIPT_SUNDANESE                     = 113,/* Sund */
  HB_SCRIPT_MOON                          = 114,/* Moon */
  HB_SCRIPT_MEITEI_MAYEK                  = 115,/* Mtei */
  
  /* New script codes from ISO 15924 @stable ICU 4.0 */
  HB_SCRIPT_IMPERIAL_ARAMAIC              = 116,/* Armi */
  HB_SCRIPT_AVESTAN                       = 117,/* Avst */
  HB_SCRIPT_CHAKMA                        = 118,/* Cakm */
  HB_SCRIPT_KOREAN                        = 119,/* Kore */
  HB_SCRIPT_KAITHI                        = 120,/* Kthi */
  HB_SCRIPT_MANICHAEAN                    = 121,/* Mani */
  HB_SCRIPT_INSCRIPTIONAL_PAHLAVI         = 122,/* Phli */
  HB_SCRIPT_PSALTER_PAHLAVI               = 123,/* Phlp */
  HB_SCRIPT_BOOK_PAHLAVI                  = 124,/* Phlv */
  HB_SCRIPT_INSCRIPTIONAL_PARTHIAN        = 125,/* Prti */
  HB_SCRIPT_SAMARITAN                     = 126,/* Samr */
  HB_SCRIPT_TAI_VIET                      = 127,/* Tavt */
  HB_SCRIPT_MATHEMATICAL_NOTATION         = 128,/* Zmth */
  HB_SCRIPT_SYMBOLS                       = 129,/* Zsym */
  
  /* Private use codes from Qaaa - Qabx are not supported*/
  HB_SCRIPT_CODE_LIMIT    = 130
};
typedef enum _hb_script_code_t hb_script_code_t;

#ifdef HB_USE_ICU_PROPERTIES
#include "uchar.h"
#define hb_get_bidi_class(ch) ((hb_bidi_class_t)u_charDirection(ch))
#endif

HB_BEGIN_DECLS

#ifndef HB_USE_ICU_PROPERTIES
hb_bidi_class_t hb_get_bidi_class(hb_codepoint_t ch);
#endif

hb_arabic_join_type_t hb_get_arabic_join_type(hb_codepoint_t ch);

hb_script_code_t hb_get_script_code(hb_codepoint_t ch);

HB_END_DECLS

#endif
