/*
 * Copyright 2009 Mozilla Corporation.
 *
 * This is part of HarfBuzz, an OpenType Layout engine library.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Created by Jonathan Kew on 21/06/2009.
 */

#ifndef HB_SHAPER_PRIVATE_H
#define HB_SHAPER_PRIVATE_H

/* internal utilities for shaper implementation */

#include "hb-common.h"
#include "hb-ot-layout.h"
#include "hb-buffer-private.h"
#include "hb-shaper.h"

#define NO_INDEX ((unsigned int) 0xFFFF)

typedef hb_bool_t (*hb_lookup_applicator_func_t) (hb_ot_layout_t *,
						  hb_buffer_t *,
						  unsigned int,
						  hb_ot_layout_feature_mask_t);

HB_BEGIN_DECLS

/*
 * For a collection of feature settings, find all the relevant lookups in a particular table_type
 * (GSUB or GPOS), and apply them using the given function.
 */
void hb_apply_features(hb_ot_layout_t * layout,
		       hb_ot_layout_table_type_t table_type,
		       unsigned int script_index,
		       unsigned int language_index,
		       const feature_setting_t * features,
		       int num_features,
		       hb_buffer_t * buffer,
		       hb_lookup_applicator_func_t apply_function);

/*
 * Merge settings from user_features with the defaults provided in orig_features, appending new features
 * or updating the setting of existing entries. If user_features is NULL (or count is 0), returns orig_features
 * unchanged, otherwise returns a newly-allocated array; use hb_free_feature_list to release that later.
 * The feature_count parameter is updated to the new count of the merged list (may be less than the sum of
 * the two input lists, if some of the user features actually replace original settings).
 */
const feature_setting_t * hb_merge_user_features(const feature_setting_t * orig_features,
						 unsigned int * feature_count,
						 const feature_setting_t * user_features,
						 unsigned int user_feature_count);

void hb_free_feature_list(const feature_setting_t * features);

HB_END_DECLS

#endif
